import os
from abc import ABC, abstractmethod
from pathlib import Path
from typing import Union, Any, Optional, Tuple
import time

import numpy as np
import pandas as pd
import cv2
from scipy import stats

from movement_detector.utils import get_video_mapped_path
from movement_detector.video import AbstractVideo


class AbstractMovementDetector(ABC):
    """Abstract class for all movement detectors.

    Contains the metadata for all frames in the video.

    Each frame's metadata contains the following fields:

    **time**: The timestamp for the frame in seconds since the start of the
    video.

    **moving**: Set to True if there is movement in the frame.

    **outlier**: Set to True if the frame is detected to be an outlier according
    to the detection method. If the detection method does not hold the concept
    of "outliers", then all fields must be set to False.

    **flagged**: Set to True for flagged outlier frames, and
    changed to False upon manual-set of the `moving` field.

    **manual_set**: Set to True if the `moving` filed has been manually
    overwritten.

    Parameters
    ----------
    video : AbstractVideo
        The video object.
    """

    _default_cols = (
        'time',
        'moving',
        'outlier',
        'flagged',
        'manual_set',
    )

    def __init__(
            self,
            video: AbstractVideo,
    ):
        self._video = video
        self._meta_path = None
        self.meta_fields = self._default_cols
        self.meta_fields += self._additional_columns
        self._meta_built = False

    @property
    def video(self) -> AbstractVideo:
        """The video associated with the detector."""
        return self._video

    @property
    def meta_built(self) -> bool:
        """Set to True if the metadata has been built."""
        return self._meta_built

    @property
    def meta_path(self) -> Path:
        """Path to metadata file."""
        if self._meta_path is None:
            self._meta_path = get_video_mapped_path(
                vid_path=self.video.vid_path,
                dir_suffix='meta',
                file_extension='.csv',
            )
        return self._meta_path

    @property
    def _additional_columns(self) -> Tuple[str]:
        """Additional meta-data columns to add to the default set.

        Must return a tuple of zero or more strings.
        """
        return tuple()

    @abstractmethod
    def _build_meta(self):
        """Overwrite this method to define the metadata generation process."""
        pass

    def run(self):
        """Process the video and extract the meta-data."""
        if os.path.exists(self.meta_path):
            self._load_meta()
        else:
            self._create_empty_meta()
            self._build_meta()
            self.save_meta()
        self._meta_built = True

    def meta(
            self,
            start: int,
            stop: Optional[int] = None,
            field: Optional[Union[str, list]] = None
    ) -> Union[pd.DataFrame, pd.Series, Any]:
        """Returns a Pandas DataFrame of the required analysis metadata.

        Parameters
        ----------
        start : int
            Start index (inclusive).
        stop : int, optional
            The end index of the query (exclusive).
        field : string or list, optional
            Name of field data.

        Returns
        -------
        pandas DataFrame, pandas Series, or any other
            Can return a Pandas DataFrame, Series, or any other type,
            depending on specified indexing.
        """
        if stop is None:
            stop = start + 1
        if field is None:
            field = self._metadata.columns
        return self._metadata.iloc[start:stop][field]

    def set_freezing(self, index: int):
        """Set metadata of specified frame to freezing.

        Frame is marked as user-verified, and flagging is removed.

        Parameters
        ----------
        index : int
            Index of frame which to set as freezing.
        """
        col_names = ['moving', 'manual_set', 'flagged']
        col_vals = [False, True, False]
        self._metadata.loc[index, col_names] = col_vals

    def set_moving(self, index: int):
        """Set metadata of specified frame to moving.

        Frame is marked as user-verified, and flagging is removed.

        Parameters
        ----------
        index : int
            Index of frame which to set as moving.
        """
        col_names = ['moving', 'manual_set', 'flagged']
        col_vals = [True, True, False]
        self._metadata.loc[index, col_names] = col_vals

    def save_meta(self):
        """Save the metadata to file.

        The file path relative to the `meta` folder is the same as the video's
        path relative to the `video` folder.
        """
        parent = self.meta_path.parent
        if not os.path.exists(parent):
            os.makedirs(parent)
        self._metadata.to_csv(self.meta_path)

    def _load_meta(self):
        self._metadata = pd.read_csv(self.meta_path)

    def _create_empty_meta(self):
        self._metadata = pd.DataFrame(
            columns=self.meta_fields,
            index=range(len(self.video)),
            dtype='float',
        )


class PixelChangeFD(AbstractMovementDetector):
    """ Pixel Change Freezing Detector

    Detects freezing based on pixel-value change ratio from one frame
    to the next.

    First, a Gaussian blur filter is applied to the images. Next, the pixels
    that have changed between the consecutive frames are detected. Then,
    bounding-boxes are built around those pixels. Finally the ratio of the area
    of the bounding boxes to the total area of the image is analyzed to
    determine if the image should be classified as one containing movement
    or not.

    The exact values for the parameters must be determined on the basis of the
    analyzed videos. The values will be very different depending on factors
    such as the size of the moving object, contrast of the object with its
    background etc.

    Parameters
    ----------
    video : AbstractVideo
        The video to analyze.
    outlier_change_threshold : float
        The number of standard deviations from the mean that the change value
        must be to be considered an outlier.
    flag_outliers_buffer : int
        The number of consecutive frames that must be identified as outliers
        in order to flag all those frames for review.
    movement_threshold : float
        A value between 0 and 1. The ratio of pixel-change areas relative to the
        area of the image above which the frame will be flagged as containing
        movement.
    freezing_buffer : int
        The number of consecutive frames that must be identified as containing
        freezing in order to the `moving` field for all those frames to False.
    blur_ksize : int
        The size of the Gaussian blur filter. For more information refer to:
        https://docs.opencv.org/master/d4/d13/tutorial_py_filtering.html

    References
    ----------
    [1] https://github.com/WillBrennan/MotionDetector
    """

    def __init__(
            self,
            video: AbstractVideo,
            outlier_change_threshold: float,
            flag_outliers_buffer: int,
            movement_threshold: float,
            freezing_buffer: int,
            blur_ksize: int,
    ):
        super().__init__(video=video)
        self.outlier_change_threshold = outlier_change_threshold
        self.flag_outliers_buffer = flag_outliers_buffer
        self.movement_threshold = movement_threshold
        self.freezing_buffer = freezing_buffer
        self.blur_ksize = blur_ksize

    @property
    def _additional_columns(self) -> Tuple[str]:
        return 'change_ratio',

    # todo: improve readability
    def _build_meta(self, _timeit: bool = False):
        prev_frame = None
        t1 = time.time() if _timeit else None
        freezing_frames = 0
        self._metadata.loc[0, 'moving'] = True
        img_area = self.video.frame_shape[0] * self.video.frame_shape[1]
        for i, frame in enumerate(self.video):
            frame = self._frame_preprocessing(frame)
            if prev_frame is None:
                change_ratio = 0
            else:
                diff = cv2.absdiff(prev_frame, frame)
                diff = self._frame_postprocessing(diff)
                contours = cv2.findContours(diff, cv2.RETR_EXTERNAL,
                                            cv2.CHAIN_APPROX_SIMPLE)[0]
                contours_area = self._get_contours_area(contours)
                change_ratio = contours_area / img_area
            self._metadata.loc[i, 'change_ratio'] = change_ratio
            if change_ratio < self.movement_threshold:
                if freezing_frames < self.freezing_buffer - 1:
                    self._metadata.loc[i, 'moving'] = True
                    freezing_frames += 1
                elif freezing_frames == self.freezing_buffer - 1:
                    for j in range(self.freezing_buffer):
                        self._metadata.loc[i - j, 'moving'] = False
                    freezing_frames += 1
                else:
                    self._metadata.loc[i, 'moving'] = False
            else:
                freezing_frames = 0
                self._metadata.loc[i, 'moving'] = True
            self._metadata.loc[i, 'manual_set'] = False
            self._metadata.loc[i, 'time'] = self.video.get_frame_time()
            prev_frame = frame
        self._metadata.loc[:, 'moving'] = self._metadata['moving'].astype(bool)
        self._metadata.loc[:, 'manual_set'] = (
            self._metadata['manual_set'].astype(bool)
        )
        self._update_meta()
        if _timeit:
            print('Video {} analyzed in {:.2f}s'.format(self.video.vid_name,
                                                        time.time() - t1))

    @staticmethod
    def _frame_postprocessing(frame: np.ndarray) -> np.ndarray:
        output = cv2.threshold(frame, 15, 255, cv2.THRESH_BINARY)[1]
        output = cv2.dilate(output, None, iterations=2)
        return output

    @staticmethod
    def _get_contours_area(contours: np.ndarray) -> float:
        area = sum([cv2.contourArea(c) for c in contours])
        return area

    def _frame_preprocessing(self, frame: np.ndarray) -> np.ndarray:
        output = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)  # convert to GreyScale
        output = cv2.GaussianBlur(
            output,
            (self.blur_ksize, self.blur_ksize),
            0
        )  # blur the image to remove high freq noise
        return output

    def _update_meta(self):
        outlier_threshold = self.outlier_change_threshold
        flag_outliers_window = self.flag_outliers_buffer
        change_zscore = np.abs(stats.zscore(self._metadata['change_ratio']))
        self._metadata.loc[:, 'outlier'] = change_zscore > outlier_threshold
        virgin_indexes = ~self._metadata['manual_set']
        # ugly, but optimized
        self._metadata.loc[virgin_indexes, 'flagged'] = (
                self._metadata['outlier'].rolling(
                    flag_outliers_window,
                    min_periods=0
                ).sum() == flag_outliers_window
        )[virgin_indexes]
        flagged_backfill = self._metadata['flagged'].copy()
        for i in range(1, flag_outliers_window + 1):
            flagged_backfill.loc[
                self._metadata['flagged'].shift(-i, fill_value=False)
            ] = True
        self._metadata.loc[:, 'flagged'] = flagged_backfill



from abc import ABC, abstractmethod
import os
from pathlib import Path
from typing import Optional

import cv2
import numpy as np

from movement_detector.np_utils import get_dtype


class AbstractVideo(ABC):
    """Interface definition for the video classes.

    This class defines the public methods that a video class must expose to
    be compatible with the movement detectors.

    Parameters
    ----------
    file_path : Path
        The path to the video file.
    """

    def __init__(self, file_path: Path):
        self.vid_path = os.path.realpath(file_path)
        self.vid_name = os.path.basename(self.vid_path)

    @property
    @abstractmethod
    def vid_duration(self) -> float:
        """The duration of the video in seconds.

        Returns
        -------
        float
        """
        pass

    @property
    @abstractmethod
    def frame_shape(self) -> tuple:
        """The shape of a single frame.

        Returns
        -------
        tuple
        """
        pass

    @property
    @abstractmethod
    def frame_rate(self) -> float:
        """The video's frame rate.

        Returns
        -------
        float
        """
        pass

    @abstractmethod
    def __iter__(self) -> iter:
        pass

    @abstractmethod
    def __next__(self) -> np.ndarray:
        pass

    @abstractmethod
    def __len__(self) -> int:
        pass

    @abstractmethod
    def __getitem__(self, item) -> np.ndarray:
        pass

    @abstractmethod
    def get_frame(self, i: int) -> np.ndarray:
        """Returns the frame at index i.

        Parameters
        ----------
        i : int
            Index of desired frame.

        Returns
        -------
        NumPy array
            The frame as a NumPy array.
        """
        pass

    @abstractmethod
    def get_frame_time(self, i: Optional[int] = None) -> float:
        """Returns the video-time for the frame i in seconds.

        Parameters
        ----------
        i : int
            Index of desired frame time.

        Returns
        -------
        float
            The time in seconds since start of video.
        """
        pass

    @abstractmethod
    def mean(self) -> np.ndarray:
        """Computes the pixel-wise mean for the frames in the video.

        Returns
        -------
        NumPy array
            Numpy array with the pixel-wise mean values.
        """
        pass

    @abstractmethod
    def std(self) -> np.ndarray:
        """Computes the pixel-wise standard deviation of the video frames.

        Returns
        -------
        NumPy
            Numpy array with the pixel-wise standard deviation values.
        """
        pass

    @abstractmethod
    def sum(self) -> np.ndarray:
        """Computes the pixel-wise sum of the frames in the video.

        Returns
        -------
        NumPy array
            Numpy array with the pixel-wise sum values.
        """
        pass


class CvVideo(AbstractVideo):
    """OpenCV implementation of the video class interface.

    OpenCV provides the most optimized algorithms for video processing.
    For efficiency, the implementation resorts to lazy evaluation and caching
    of the compute-intensive operations.
    """
    _precision_dtype = np.dtype('float32')

    def __init__(self, file_path: Path):
        super().__init__(file_path)
        self._frames = cv2.VideoCapture(self.vid_path)
        self._frame_count = int(self._frames.get(cv2.CAP_PROP_FRAME_COUNT))
        if self._frame_count <= 0:
            raise ValueError(
                f'The video appears to be empty. Path: {self.vid_path}.'
            )
        vid_height = int(self._frames.get(cv2.CAP_PROP_FRAME_HEIGHT))
        vid_width = int(self._frames.get(cv2.CAP_PROP_FRAME_WIDTH))
        self._frame_shape = (vid_height, vid_width, 3)
        self._frame_rate = self._frames.get(cv2.CAP_PROP_FPS)
        self._vid_duration = self._frame_count / self._frame_rate
        self._current_frame = 0
        self._sum: np.ndarray = None
        self._mean: np.ndarray = None
        self._std: np.ndarray = None
        # fix the problem where open CV returns one more than
        # the actual video length
        while self.get_frame(self._frame_count-1) is None:
            self._frame_count -= 1
        self.size = int(np.prod(self._frame_shape + (self._frame_count, 8)))

    @property
    def vid_duration(self) -> float:
        return self._vid_duration

    @property
    def frame_shape(self) -> tuple:
        return self._frame_shape

    @property
    def frame_rate(self) -> float:
        return self._frame_rate

    def __iter__(self) -> iter:
        self._frames.set(cv2.CAP_PROP_POS_FRAMES, 0)
        return self

    def __next__(self) -> np.ndarray:
        ret, frame = self._frames.read()
        if ret:
            return frame
        else:
            self._current_frame = self._frame_count
            raise StopIteration

    def __len__(self) -> int:
        return self._frame_count

    def __getitem__(self, item) -> np.ndarray:
        if type(item) is slice:
            indices = range(*item.indices(self._frame_count))
            frames = np.empty(
                shape=(len(indices),) + self.frame_shape,
                dtype=np.uint8
            )
            # if the indices are sequential, use optimized retrieval
            if item.step in [None, 1]:
                self._frames.set(cv2.CAP_PROP_POS_FRAMES, indices[0])
                frames = np.array([self._frames.read()[1] for _ in indices])
                self._current_frame = indices[-1]
            else:
                for i in range(len(indices)):
                    frames[i] = self.get_frame(indices[i])
            return frames
        elif item >= self._frame_count:
            raise IndexError('Index out of range')
        else:
            return self.get_frame(item)

    def sum(self) -> np.ndarray:
        if self._sum is None:
            max_pixel_value = 255 * self._frame_count
            dtype = get_dtype(max_pixel_value)
            self._sum = np.zeros(self.frame_shape, dtype=dtype)
            self._frames.set(cv2.CAP_PROP_POS_FRAMES, 0)
            for index in range(self._frame_count):
                self._sum += self._frames.read()[1]
            self._current_frame = self._frame_count
        return self._sum

    def mean(self) -> np.ndarray:
        if self._mean is None:
            mean = self.sum() / self._frame_count
            self._mean = mean.astype(self._precision_dtype)
        return self._mean

    def std(self) -> np.ndarray:
        if self._std is None:
            squares_sum = np.zeros(
                shape=self.frame_shape,
                dtype=self._precision_dtype,
            )
            self._frames.set(cv2.CAP_PROP_POS_FRAMES, 0)
            for index in range(self._frame_count):
                frame = self._frames.read()[1]
                mean_diff = frame - self.mean()
                squares_sum += np.square(mean_diff)
            self._std = np.sqrt(squares_sum / self._frame_count)
            self._current_frame = self._frame_count
        return self._std

    def get_frame(self, i: Optional[int] = None) -> np.ndarray:
        if i is not None and self._current_frame != i:
            self._frames.set(cv2.CAP_PROP_POS_FRAMES, i)
            self._current_frame = i
        _, frame = self._frames.read()
        self._current_frame += 1
        return frame

    def get_frame_time(self, i: Optional[int] = None) -> float:
        if i is not None:
            self.get_frame(i=i)
        frame_time = self._frames.get(cv2.CAP_PROP_POS_MSEC) / 1000
        return frame_time



import os
from abc import ABC, abstractmethod
from pathlib import Path
from typing import List, Callable

import pandas as pd
import numpy as np

from movement_detector.detectors import AbstractMovementDetector
from movement_detector.utils import get_video_mapped_path


class AbstractMetaAnalyzer(ABC):
    """Metadata analyzer.

    A group of classes that can analyze metadata produced by a detector object.

    Parameters
    ----------
    detector : AbstractMovementDetector
        The detector who's metadata to analyze.
    """

    def __init__(self, detector: AbstractMovementDetector):
        self.detector = detector
        # TODO: this violates the open-closed principle
        self.analysis_path = self.get_analysis_path(
            vid_path=self.detector.video.vid_path
        )

    def run(self):
        """Run the analysis.

        Applies the analysis to the metadata and saves the results
        as a CSV file.

        If the detector hasn't been ran, `self.detector.run()` is called.
        """
        if not os.path.exists(self.analysis_path):
            if not self.detector.meta_built:
                self.detector.run()
            analysis = self._analyze_meta(df=self.detector._metadata)
            self._save_analysis(analysis=analysis)
        else:
            analysis = self._load_analysis()
        return analysis

    @staticmethod
    def get_analysis_path(vid_path: Path) -> Path:
        """Returns the path in which to save the analysis file.

        The analysis is saved in a sub-path relative to the analysis folder
        that mimics the sub-path of the detector's video file relative to the
        videos folder.

        Parameters
        ----------
        vid_path : Path
            The path to the video file.

        Returns
        -------
        path : Path
            The path to the analysis file.
        """
        path = get_video_mapped_path(
            vid_path=vid_path,
            dir_suffix='analysis',
            file_extension='.csv',
        )
        return path

    @abstractmethod
    def _analyze_meta(self, df: pd.DataFrame):
        """Overwrite to implement the meta-analysis.

        Should accept a Pandas dataframe produced by a detector object. Refer
        to the documentation of the detector class for more details on the
        available fields.
        """
        pass

    def _save_analysis(self, analysis):
        parent = self.analysis_path.parent
        if not os.path.exists(parent):
            os.makedirs(parent)
        analysis.to_csv(self.analysis_path)

    def _load_analysis(self):
        analysis = pd.read_csv(self.analysis_path)
        return analysis


class IntervalAggregatorMA(AbstractMetaAnalyzer):
    """Interval metadata aggregator.

    The interval aggregator applies an aggregation function to the count
    of the frames containing movement flagged by the detector over the
    specified intervals of time.

    Passing an `np.mean` aggregation will result in the percentage of movement
    frames detected in each interval.

    Parameters
    ----------
    detector : AbstractMovementDetector
        The detector who's metadata to analyze.
    intervals : list of floats
        A list of the interval cut-off points.
    aggregation : Callable, default np.mean
        The aggregation operation that will be applied on the movement-positive
        frames in the specified intervals.
    include_start : bool, default True
        If set to True and the first cut-off point is not zero, the first
        interval will span from the start of the video to the first cut-off.
    include_end : bool, default True
        If set to True and the last cut-off point is lower than the duration
        of the video, the last interval will span from the cut-off point to
        the end of the video.
    """

    def __init__(
            self,
            detector: AbstractMovementDetector,
            intervals: List[float],
            aggregation: Callable = np.mean,
            include_start: bool = True,
            include_end: bool = True,
    ):
        super().__init__(detector=detector)
        self._intervals = intervals
        self._aggregation = aggregation
        self._include_start = include_start
        self._include_end = include_end

    @property
    def intervals(self) -> List[float]:
        """The list of cut-off points for the intervals."""
        return self._intervals

    @property
    def aggregation(self) -> Callable:
        """The aggregation operation."""
        return self._aggregation

    def _analyze_meta(self, df: pd.DataFrame):
        if self._include_start:
            first_timestamp = self.intervals[0]
            if first_timestamp != 0:
                self.intervals.insert(0, 0)
        if self._include_end:
            last_timestamp = self.intervals[-1]
            vid_len = self.detector.video.vid_duration
            if vid_len > last_timestamp:
                self.intervals.append(vid_len)

        df = df.groupby(pd.cut(df['time'], bins=self.intervals))
        df = df.aggregate(self._aggregation)['moving']
        return df



import numpy as np


def get_dtype(value: np.number):
    """
    Returns the largest numpy value-type capable of representing the passed-in
    value. If the largest float needed, value must be a float. If signed numbers
    are needed, value must be passed as negative (note that floats are always
    signed).

    Parameters
    ----------
    value : numeric value
        The value to be represented.

    Returns
    -------
    NumPy dtype
    """
    np_floats = [np.float16, np.float32, np.float64, np.float128]
    np_uints = [np.uint8, np.uint16, np.uint32, np.uint64]
    np_ints = [np.int8, np.int16, np.int32, np.int64]
    if isinstance(value, float):
        if value > 0:
            for float_ in np_floats:
                if np.finfo(float_).max > value:
                    return float_
        else:
            for float_ in np_floats:
                if np.finfo(float_).min < value:
                    return float_
    elif value > 0:
        for uint_ in np_uints:
            if np.iinfo(uint_).max > value:
                return uint_
    else:
        for int_ in np_ints:
            if np.iinfo(int_).min < value:
                return int_



import os
from pathlib import Path
from typing import List


def get_project_path() -> Path:
    """Get the path to the project's root directory.

    Returns
    -------
    project_path : Path
        The path to the root directory of the project.
    """
    file_dir = os.path.dirname(os.path.realpath(__file__))
    dir_path = Path(file_dir)
    project_path = dir_path.parent
    return project_path


def check_vid_path(vid_path):
    vid_path = os.path.realpath(vid_path)
    videos_path = str(get_project_path() / 'videos')
    valid = True
    if len(vid_path) < len(videos_path):
        valid = False
    elif vid_path[:len(videos_path)] != videos_path:
        valid = False
    elif not os.path.isfile(vid_path):
        valid = False
    if not valid:
        raise ValueError(f'Invalid video path {vid_path}.')


def get_video_mapped_path(vid_path, dir_suffix, file_extension):
    check_vid_path(vid_path=vid_path)
    vid_path = os.path.realpath(vid_path)
    videos_path = get_project_path() / 'videos'
    vid_relative_path = vid_path[len(str(videos_path)):]
    data_path = get_project_path() / f'{dir_suffix}{vid_relative_path}'
    data_path = data_path.with_suffix(file_extension)
    return data_path


def get_video_paths() -> List[Path]:
    videos_path = get_project_path() / 'videos'
    vid_paths = []
    for root, dirs, files in os.walk(videos_path):
        for file in files:
            path = os.path.join(root, file)
            if _is_video_file(path=path):
                vid_paths.append(path)
    return vid_paths


def _is_video_file(path) -> bool:
    path = str(path)
    valid = True
    if not os.path.isfile(path):
        valid = False
    elif not (path.endswith(('.mp4', '.wmv'))):
        valid = False
    return valid



from movement_detector.video import CvVideo
from movement_detector.detectors import PixelChangeFD
from movement_detector.analysis import IntervalAggregatorMA

__all__ = [
    'CvVideo',
    'PixelChangeFD',
    'IntervalAggregatorMA',
]



from os import path
from pathlib import Path
import re

from setuptools import setup, find_packages

NAME = 'movement-detector'
VERSION = 'v0.2.4'
DESCRIPTION = 'Detect movement in videos.'
THIS_DIR = Path(path.abspath(path.dirname(__file__)))
INSTALL_PACKAGES = [
    'numpy==1.18.1',
    'pandas==1.0.1',
    'scipy==1.4.1',
    'opencv-python==4.1.2.30',
    'moviepy==1.0.1',
    'imageio==2.6.1',
]
INTERFACE_PACKAGES = [
    'pygame==1.9.6',
]
DEV_PACKAGES = [
    'PyYAML>=5.1',
    'pytest==5.4.1',
    'sphinx==2.4.4',
    'sphinx_rtd_theme==0.4.3',
    'numpydoc==0.9.2',
    'sphinxcontrib-napoleon==0.7',
    'twine',
    'bumpversion',
]
APP = ['main.py']
DATA_FILES = []
OPTIONS = {}


def fix_readme(txt):
    return re.sub(r'\.\. PYPI-BEGIN([\r\n]|.)*?PYPI-END', '', txt, re.DOTALL)


with open(THIS_DIR / 'README.md', encoding='utf-8') as f:
    LONG_DESCRIPTION = fix_readme(f.read())

with open(THIS_DIR / 'LICENSE.txt', encoding='utf-8') as f:
    LICENSE = f.read()


setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    # url={'Documentation': '', 'Source': ''},
    author='Petio Petrov',
    license=LICENSE,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Science/Research',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    keywords='movement detection freezing behavioural behaviour behavioral'
             'behavior research',
    app=APP,
    data_files=DATA_FILES,
    options={'py2app': OPTIONS},
    setup_requires=['py2app'],
    install_requires=INSTALL_PACKAGES,
    extras_require={
        'dev': DEV_PACKAGES,
        'interface': INTERFACE_PACKAGES,
    },
    # package_dir={'': ''},
    packages=find_packages(
        include=['movement_detector', 'movement_detector.*'],
    ),
    url='https://github.com/petioptrv/movement_detector',
)
